#include <stdint.h>       // Find types like "uint8_t"
#include <stdbool.h>      // Find type "bool"
#include <SPI.h>          // Arduino SPI library
#include <SD.h>           // Arduino SD card library

#include "Eve2_81x.h"     // Matrix Orbital Eve2 Library      
#include "Display_Timings.h" // TFT timings
#include "Arduino_AL.h"   // Hardware abstraction layer for Arduino
#include "process.h"      // More application level code
#include "touch_cap_811.h"      // More application level code


// The file of type File and named myFile is global and used for any and all files.  
// Your job is to make sure you are not trying to open it twice.
File myFile;

// A global buffer for use with debug messages.  It is always used up immediately so 
// never retains initialized values.  Prevent further allocations of RAM for logging
char LogBuf[LogBufSize];
byte CapCalibrated = 0;

void setup()
{
  // Initializations.  Order is important
  GlobalInit();
  FT81x_Init();
  SD_Init();

  CapCalibrated = Calibrate_Cap_Touch();
  //CapCalibrated = MO_Calibrate();
  //wr16(REG_TOUCH_CONFIG + RAM_REG, 0x05D0);             // Only available on BT815 units (EVE3)
  MyDelay(1000);
}

void loop()
{         
  uint16_t touchX = 0, touchY = 0; 
  uint32_t touchValue = 0;
  uint8_t tag = 0;    
  char charBuffer[17];

  while(CapCalibrated) 
  {  
      Send_CMD(CMD_DLSTART);
      Send_CMD(CLEAR_COLOR_RGB(0,0,0));
      Send_CMD(CLEAR(1,1,1));
      Send_CMD(COLOR_RGB(255, 0, 0));
      Send_CMD(POINT_SIZE(10 * 16));
      Send_CMD(BEGIN(POINTS));
      Send_CMD(TAG(1));
      Send_CMD(VERTEX2F((10) * 16, (11) * 16));
      Send_CMD(TAG(2));
      Send_CMD(VERTEX2F((469) * 16, (58) * 16));
      Send_CMD(TAG(3));
      Send_CMD(VERTEX2F((240) * 16, (106) * 16));  
      Cmd_Text(240, 58, 27, OPT_CENTERX, "Tap on the dots");  
      Cmd_Text(10, 60, 22, 0, "Dot");    
      Cmd_Text(10, 80, 22, 0, "XCoordinates");  
      Cmd_Text(10, 100, 22, 0, "YCoordinates");  
      itoa(tag, charBuffer, 10);
      Cmd_Text(40, 60, 22, 0, charBuffer);  
      itoa(touchX, charBuffer, 10);
      Cmd_Text(120, 80, 22, 0, charBuffer);  
      itoa(touchY, charBuffer, 10);
      Cmd_Text(120, 100, 22, 0, charBuffer);  
      Send_CMD(DISPLAY());
      Send_CMD(CMD_SWAP);
      UpdateFIFO();                                               // Trigger the CoProcessor to start processing commands out of the FIFO
      
      Wait4CoProFIFOEmpty();                                      // wait here until the coprocessor has read and executed every pending command.  
  
     //touchValue = rd32(RAM_REG + REG_TOUCH_DIRECT_XY);// MO: Read for any new touch tag inputs
     tag = rd8(REG_CTOUCH_TAG + RAM_REG);                    // Check for touches
     Log("%u\n",tag);   
     touchValue = rd32(RAM_REG + REG_CTOUCH_TOUCH_XY);// MO: Read for any new touch tag inputs
     Log("%d\n",touchValue);        
     if (!(touchValue & 0x80000000))
     {
      touchY = touchValue & 0x03FF; //Raw Touchscreen Y coordinate
      touchX = (touchValue >> 16) & 0x03FF; //Raw Touchscreen Y coordinate        
     }         
     MyDelay(10);
  }
}


int8_t MO_Calibrate(void)
{
  uint8_t CTOUCH_CONFIG_DATA_G911[] = { TOUCH_DATA_U8 };  
  WriteBlockRAM(RAM_G,CTOUCH_CONFIG_DATA_G911, TOUCH_DATA_LEN);
  wr8(RAM_REG + REG_CPU_RESET, 2);  
  wr16(RAM_REG + REG_CYA_TOUCH, 0x05D0);  
  wr8(RAM_REG + REG_GPIOX_DIR, (rd8(RAM_REG +  REG_GPIOX_DIR) | 0x08));
  wr8(RAM_REG + REG_GPIOX, (rd8(RAM_REG +  REG_GPIOX) & 0xF7));  
  MyDelay(1);  
  wr8(RAM_REG + REG_CPU_RESET, 0);  
  MyDelay(100);  
  wr8(RAM_REG + REG_GPIOX_DIR, (rd8(RAM_REG +  REG_GPIOX_DIR) & 0xF7));  
  return 1;
}

// ************************************************************************************
// Following are wrapper functions for C++ Arduino functions so that they may be      *
// called from outside of C++ files.  These are also your opportunity to use a common *
// name for your hardware functions - no matter the hardware.  In Arduino-world you   *
// interact with hardware using Arduino built-in functions which are all C++ and so   *
// your "abstraction layer" must live in this xxx.ino file where C++ works.           *
//                                                                                    *
// Incidentally, this is also an alternative to ifdef-elif hell.  A different target  *
// processor or compiler will include different files for hardware abstraction, but   *
// the core "library" files remain unaltered - and clean.  Applications built on top  *
// of the libraries need not know which processor or compiler they are running /      *
// compiling on (in general and within reason)                                        *
// ************************************************************************************

void GlobalInit(void)
{
  Serial.begin(115200);
  while (!Serial) {;}                    // wait for serial port to connect.
  
  // Keys Initialization
  Init_Keys();

  // Matrix Orbital Eve display interface initialization
  pinMode(EvePDN_PIN, OUTPUT);            // Pin setup as output for Eve PDN pin.
  digitalWrite(EvePDN_PIN, LOW);          // Apply a resetish condition on Eve
  pinMode(EveChipSelect_PIN, OUTPUT);     // SPI CS Initialization
  digitalWrite(EveChipSelect_PIN, HIGH);  // Deselect Eve
  pinMode(EveAudioEnable_PIN, OUTPUT);    // Audio Enable PIN
  digitalWrite(EveAudioEnable_PIN, LOW);  // Disable Audio
  digitalWrite(EvePDN_PIN, HIGH);          // Apply a resetish condition on Eve
  SPI.begin();                            // Enable SPI
  
}

// Send a single byte through SPI
void SPI_WriteByte(uint8_t data)
{
  SPI.beginTransaction(SPISettings(100000, MSBFIRST, SPI_MODE0));
  digitalWrite(EveChipSelect_PIN, LOW);

  SPI.transfer(data);
      
  digitalWrite(EveChipSelect_PIN, HIGH);
  SPI.endTransaction();
}

// Send a series of bytes (contents of a buffer) through SPI
void SPI_WriteBuffer(uint8_t *Buffer, uint32_t Length)
{
  SPI.beginTransaction(SPISettings(100000, MSBFIRST, SPI_MODE0));
  digitalWrite(EveChipSelect_PIN, LOW);

  SPI.transfer(Buffer, Length);
      
  digitalWrite(EveChipSelect_PIN, HIGH);
  SPI.endTransaction();
}

// Send a byte through SPI as part of a larger transmission.  Does not enable/disable SPI CS
void SPI_Write(uint8_t data)
{
//  Log("W-0x%02x\n", data);
  SPI.transfer(data);
}

// Read a series of bytes from SPI and store them in a buffer
void SPI_ReadBuffer(uint8_t *Buffer, uint32_t Length)
{
  uint8_t a = SPI.transfer(0x00); // dummy read

  while (Length--)
  {
    *(Buffer++) = SPI.transfer(0x00);
  }
}

// Enable SPI by activating chip select line
void SPI_Enable(void)
{
  SPI.beginTransaction(SPISettings(100000, MSBFIRST, SPI_MODE0));
  digitalWrite(EveChipSelect_PIN, LOW);
}

// Disable SPI by deasserting the chip select line
void SPI_Disable(void)
{
  digitalWrite(EveChipSelect_PIN, HIGH);
  SPI.endTransaction();
}

void Init_Keys(void)
{
  // Pin setup as input for each button pin
  pinMode(Button1_PIN, INPUT);
  pinMode(Button2_PIN, INPUT);
  pinMode(Button3_PIN, INPUT);
}

void Eve_Reset_HW(void)
{
  // Reset Eve
  SetPin(EvePDN_PIN, 0);                    // Set the Eve PDN pin low
  MyDelay(50);                             // delay
  SetPin(EvePDN_PIN, 1);                    // Set the Eve PDN pin high
  MyDelay(100);                            // delay
}

void DebugPrint(char *str)
{
  Serial.print(str);
}

// A millisecond delay wrapper for the Arduino function
void MyDelay(uint32_t DLY)
{
  uint32_t wait;
  wait = millis() + DLY; while(millis() < wait);
}

// An abstracted pin write that may be called from outside this file.
void SetPin(uint8_t pin, bool state)
{
  digitalWrite(pin, state); 
}

// An abstracted pin read that may be called from outside this file.
uint8_t ReadPin(uint8_t pin)
{
  return(digitalRead(pin));
}


void SD_Init(void)
{
//  Log("Initializing SD card...\n");
  if (!SD.begin(SDChipSelect_PIN)) 
  {
    Log("SD initialization failed!\n");
    return;
  }
  Log("SD initialization done\n");
}

// ************************************************************************************
// Following are abstracted file operations for Arduino.  This is possible by using a * 
// global pointer to a single file.  It is enough for our needs and it hides file     *
// handling details within the abstraction.                                           *
// ************************************************************************************
void FileOpen(char *filename, uint8_t mode)
{
  // Since one also loses access to defined values like FILE_READ from outside the .ino
  // I have been forced to make up values and pass them here (mode) where I can use the 
  // Arduino defines.
  switch(mode)
  {
  case FILEREAD:
    myFile = SD.open(filename, FILE_READ);
    break;
  case FILEWRITE:
    myFile = SD.open(filename, FILE_WRITE);
    break;
  default:;
  }
}

void FileClose(void)
{
  myFile.close();
  if(myFileIsOpen())
  {
    Log("Failed to close file\n");
  }
}

// Read a single byte from a file
uint8_t FileReadByte(void)
{
  return(myFile.read());
}

// Read bytes from a file into a provided buffer
void FileReadBuf(uint8_t *data, uint32_t NumBytes)
{
  myFile.read(data, NumBytes);
}

void FileWriteByte(uint8_t data)
{
  myFile.write(data);
}

uint32_t FileSize(void)
{
  return(myFile.size());
}

uint32_t FilePosition(void)
{
  return(myFile.position());
}

bool FileSeek(uint32_t offset)
{
  return(myFile.seek(offset));
}

bool myFileIsOpen(void)
{
  if(myFile)
    return true;
  else
    return false;
}
