// Process.c is the application layer.  All function calls are hardware ambivalent.

#include <stdint.h>              // Find integer types like "uint8_t"  
#include "Eve2_81x.h"            // Matrix Orbital Eve2 Driver
#include "Arduino_AL.h"          // include the hardware specific abstraction layer header for the specific hardware in use.
#include "process.h"             // Every c file has it's header and this is the one for this file
#include "Display_Timings.h" // TFT timings

// A calibration screen for the touch digitizer
void MakeScreen_Calibrate(void)
{
  
  Send_CMD(CMD_DLSTART);
  Send_CMD(CLEAR_COLOR_RGB(0,0,0));
  Send_CMD(CLEAR(1,1,1));
  Cmd_Text(100, 240, 27, OPT_CENTER, "Tap on the dots");
  Cmd_Calibrate(0);                                           // This widget generates a blocking screen that doesn't unblock until 3 dots have been touched
  Send_CMD(DISPLAY());
  Send_CMD(CMD_SWAP);
  UpdateFIFO();                                               // Trigger the CoProcessor to start processing commands out of the FIFO
  
  Wait4CoProFIFOEmpty();                                      // wait here until the coprocessor has read and executed every pending command.
  MyDelay(100);
}

// This define is for the size of the buffer we are going to use for data transfers.  It is 
// sitting here so uncomfortably because it is a silly tiny buffer in Arduino Uno and you
// will want a bigger one if you can get it.  Redefine this and add a nice buffer to Load_ZLIB()
#define COPYBUFSIZE WorkBuffSz

int8_t Calibrate_Cap_Touch(void)
{
  uint32_t Remaining;
  uint16_t ReadBlockSize = 0;
  uint32_t Add_CMD = RAM_CMD;
  uint8_t SD_data = 0;
  uint32_t Remaining_Counter = 0;

  // Open the file on SD card by name
  FileOpen("Cap_Cal.bin", FILEREAD);
  if(!myFileIsOpen())
  {
    Log("Cap_Cal.bin not open\n");
    Log("Please add file to SD card\n");
    FileClose();
    return 0;
  }
  Log("Loaded\n");
  
  Remaining = FileSize();                                    // Store the size of the currently opened file
   while (Remaining)
  {
    if (Remaining > COPYBUFSIZE)
      ReadBlockSize = COPYBUFSIZE;
    else
      ReadBlockSize = Remaining;    
    Log("Block Begin\n");
    FileReadBuf(LogBuf, ReadBlockSize);                        // Read a block of data from the file
    //Add_CMD = WriteBlockRAM(Add_CMD, LogBuf, ReadBlockSize); 
    CoProWrCmdBuf(LogBuf, ReadBlockSize);
    Remaining -= ReadBlockSize;                                // Reduce remaining data value by amount just read
    Log("Block End\n");
    UpdateCMDFIFO();
    Log("Waiting End\n");
    Wait4CoProFIFOEmpty(); 
  }    
  FileClose();
  Log("Close File");

  wr8(RAM_REG + REG_CPU_RESET,2);
  wr16(RAM_REG + REG_CYA_TOUCH, 0x05D0);
  wr8(RAM_REG + REG_GPIOX_DIR, (rd8(RAM_REG + REG_GPIOX_DIR) | 0x08));
  wr8(RAM_REG + REG_GPIOX, (rd8(RAM_REG + REG_GPIOX) & 0xF7));
  MyDelay(1);
  wr8(RAM_REG + REG_CPU_RESET,0);
  MyDelay(100);
  wr8(RAM_REG + REG_GPIOX_DIR, (rd8(RAM_REG + REG_GPIOX_DIR) & 0xF7));
  /*MyDelay(100);
  wr8(RAM_REG + REG_CTOUCH_MODE,3);
  MyDelay(100);
  wr8(RAM_REG + REG_CTOUCH_EXTEND,0);  
  MyDelay(100);*/
  return 1;
}

// Is there a physical key press?
uint8_t CheckKeys(void)
{
  if(!ReadPin(Button1_PIN))
    return(1);
  if(!ReadPin(Button2_PIN))
    return(2);
  if(!ReadPin(Button3_PIN))
    return(3);

  return(0);
}
